﻿using ProjectByldozer.Drawnings;
using ProjectByldozer.MovementStrategy;
using ProjectSportCar.MovementStrategy;

namespace ProjectByldozer;
/// <summary>
/// форма рыботы с обьектом "бульдозер"
/// </summary>
public partial class FormByldozer : Form
{
    /// <summary>
    /// поле-объект для прорисовки обьекта
    /// </summary>
    private DrawningTrackedCar? _drawningTrackedCar;

    /// <summary>
    /// стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;
    /// <summary>
    /// инциализация формы
    /// </summary>
    public FormByldozer()
    {
        InitializeComponent();
        _strategy = null;
    }
    /// <summary>
    /// метод прорисовки 
    /// </summary>
    private void Draw()
    {
        if (_drawningTrackedCar == null)
        {
            return;
        }
        Bitmap bmp = new(pictureBoxByldozer.Width, pictureBoxByldozer.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningTrackedCar.DrawTransport(gr);
        pictureBoxByldozer.Image = bmp;

    }


    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создаваемого объекта</param>
    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningTrackedCar):
                _drawningTrackedCar = new DrawningTrackedCar(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningByldozer):
                _drawningTrackedCar = new DrawningByldozer(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                    Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawningTrackedCar.SetPictureSize(pictureBoxByldozer.Width, pictureBoxByldozer.Height);
        _drawningTrackedCar.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }
    /// <summary>
    /// обработка нажатия кнопки "создать бульдозер"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateByldozer_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningByldozer));

    /// <summary>
    /// обработка нажатия кнопки "создать машину"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreatecar_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningTrackedCar));

    /// <summary>
    /// премещение объекта по форме
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningTrackedCar == null)
        {
            return;
        }
        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningTrackedCar.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningTrackedCar.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningTrackedCar.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningTrackedCar.MoveTransport(DirectionType.Right);
                break;
        }
        if (result)
        {
            Draw();
        }
    }
    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningTrackedCar == null)
        {
            return;
        }
        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableTrackedCar(_drawningTrackedCar),
            pictureBoxByldozer.Width, pictureBoxByldozer.Height);
        }
        if (_strategy == null)
        {
            return;
        }
        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();
        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }

    private void pictureBoxByldozer_Click(object sender, EventArgs e)
    {

    }
}

